import tempfile
import shutil
import os
import inspect
import re
from lib import BaseTest


def gpgRemove(_, s):
    return re.sub(r'Signature made .* using|gpgv: keyblock resource .*$|gpgv: Can\'t check signature: .*$', '', s, flags=re.MULTILINE)


def changesRemove(_, s):
    return s.replace(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"), "")


def tempDirRemove(self, s):
    return s.replace(self.tempSrcDir, "")


class IncludeRepo1Test(BaseTest):
    """
    include packages to local repo: .changes file from directory
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -no-remove-files -keyring=${files}/aptly.pub ${changes}"
    outputMatchPrepare = gpgRemove

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages unstable", "repo_show")

        # check pool
        self.check_exists('pool/66/83/99580590bf1ffcd9eb161b6e5747_hardlink_0.2.1_amd64.deb')
        self.check_exists('pool/c0/d7/458aa2ca3886cd6885f395a289ef_hardlink_0.2.1.dsc')
        self.check_exists('pool/4d/f0/adce005526a1f0e1b38171ddb1f0_hardlink_0.2.1.tar.gz')


class IncludeRepo2Test(BaseTest):
    """
    include packages to local repo: .changes file from file + custom repo
    """
    fixtureCmds = [
        "aptly repo create my-unstable",
        "aptly repo add my-unstable ${files}",
    ]
    runCmd = "aptly repo include -no-remove-files -keyring=${files}/aptly.pub -repo=my-{{.Distribution}} ${changes}/hardlink_0.2.1_amd64.changes"
    outputMatchPrepare = gpgRemove

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly repo show -with-packages my-unstable", "repo_show")

        # check pool
        self.check_exists('pool/66/83/99580590bf1ffcd9eb161b6e5747_hardlink_0.2.1_amd64.deb')
        self.check_exists('pool/c0/d7/458aa2ca3886cd6885f395a289ef_hardlink_0.2.1.dsc')
        self.check_exists('pool/4d/f0/adce005526a1f0e1b38171ddb1f0_hardlink_0.2.1.tar.gz')


class IncludeRepo3Test(BaseTest):
    """
    include packages to local repo: broken repo flag
    """
    fixtureCmds = [
    ]
    runCmd = "aptly repo include -no-remove-files -keyring=${files}/aptly.pub -repo=my-{{.Distribution} ${changes}"
    expectedCode = 1

    def outputMatchPrepare(_, s):
        return s.replace('; missing space?', '')


class IncludeRepo4Test(BaseTest):
    """
    include packages to local repo: missing repo
    """
    fixtureCmds = [
    ]
    runCmd = "aptly repo include -no-remove-files -ignore-signatures -keyring=${files}/aptly.pub ${changes}"
    outputMatchPrepare = changesRemove
    expectedCode = 1


class IncludeRepo5Test(BaseTest):
    """
    include packages to local repo: remove files being added
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -keyring=${files}/aptly.pub "
    outputMatchPrepare = gpgRemove

    def prepare(self):
        super(IncludeRepo5Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()

        shutil.copytree(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"), os.path.join(self.tempSrcDir, "01"))

        shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "pyspi_0.6.1-1.3.diff.gz"),
                    os.path.join(self.tempSrcDir, "01", "pyspi_0.6.1-1.3.diff.gz"))

        self.runCmd += self.tempSrcDir

    def check(self):
        try:
            self.check_output()
            self.check_cmd_output("aptly repo show -with-packages unstable", "repo_show")

            # check pool
            self.check_exists('pool/66/83/99580590bf1ffcd9eb161b6e5747_hardlink_0.2.1_amd64.deb')
            self.check_exists('pool/c0/d7/458aa2ca3886cd6885f395a289ef_hardlink_0.2.1.dsc')
            self.check_exists('pool/4d/f0/adce005526a1f0e1b38171ddb1f0_hardlink_0.2.1.tar.gz')

            for path in ["hardlink_0.2.1.dsc", "hardlink_0.2.1.tar.gz", "hardlink_0.2.1_amd64.changes", "hardlink_0.2.1_amd64.deb"]:
                path = os.path.join(self.tempSrcDir, "01", path)
                if os.path.exists(path):
                    raise Exception("path %s shouldn't exist" % (path, ))

            path = os.path.join(self.tempSrcDir, "01", "pyspi_0.6.1-1.3.diff.gz")
            if not os.path.exists(path):
                raise Exception("path %s doesn't exist" % (path, ))

        finally:
            shutil.rmtree(self.tempSrcDir)


class IncludeRepo6Test(BaseTest):
    """
    include packages to local repo: missing files
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -keyring=${files}/aptly.pub "
    expectedCode = 1

    def outputMatchPrepare(self, s):
        return gpgRemove(self, tempDirRemove(self, s))

    def prepare(self):
        super(IncludeRepo6Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()
        os.makedirs(os.path.join(self.tempSrcDir, "01"), 0o755)

        for path in ["hardlink_0.2.1.dsc", "hardlink_0.2.1_amd64.changes", "hardlink_0.2.1_amd64.deb"]:
            shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes", path),
                        os.path.join(self.tempSrcDir, "01", path))

        self.runCmd += self.tempSrcDir

    def check(self):
        try:
            super(IncludeRepo6Test, self).check()

            for path in ["hardlink_0.2.1.dsc", "hardlink_0.2.1_amd64.changes", "hardlink_0.2.1_amd64.deb"]:
                path = os.path.join(self.tempSrcDir, "01", path)
                if not os.path.exists(path):
                    raise Exception("path %s doesn't exist" % (path, ))
        finally:
            shutil.rmtree(self.tempSrcDir)


class IncludeRepo7Test(BaseTest):
    """
    include packages to local repo: wrong checksum
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -keyring=${files}/aptly.pub "
    expectedCode = 1

    def outputMatchPrepare(self, s):
        return gpgRemove(self, tempDirRemove(self, s))

    def prepare(self):
        super(IncludeRepo7Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()

        shutil.copytree(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"), os.path.join(self.tempSrcDir, "01"))

        with open(os.path.join(self.tempSrcDir, "01", "hardlink_0.2.1.dsc"), "w") as f:
            f.write("A" * 949)  # file size

        self.runCmd += self.tempSrcDir

    def check(self):
        try:
            super(IncludeRepo7Test, self).check()
        finally:
            shutil.rmtree(self.tempSrcDir)


class IncludeRepo8Test(BaseTest):
    """
    include packages to local repo: wrong signature
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -keyring=${files}/aptly.pub "
    expectedCode = 1

    def outputMatchPrepare(self, s):
        return gpgRemove(self, tempDirRemove(self, s))

    def prepare(self):
        super(IncludeRepo8Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()

        shutil.copytree(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"), os.path.join(self.tempSrcDir, "01"))

        with open(os.path.join(self.tempSrcDir, "01", "hardlink_0.2.1_amd64.changes"), "r+") as f:
            contents = f.read()
            f.seek(0, 0)
            f.write(contents.replace('Julian', 'Andrey'))
            f.truncate()

        self.runCmd += self.tempSrcDir

    def check(self):
        try:
            super(IncludeRepo8Test, self).check()
        finally:
            shutil.rmtree(self.tempSrcDir)


class IncludeRepo9Test(BaseTest):
    """
    include packages to local repo: unsigned
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -keyring=${files}/aptly.pub "
    expectedCode = 1

    def outputMatchPrepare(self, s):
        return gpgRemove(self, tempDirRemove(self, s))

    def prepare(self):
        super(IncludeRepo9Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()

        shutil.copytree(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"), os.path.join(self.tempSrcDir, "01"))

        with open(os.path.join(self.tempSrcDir, "01", "hardlink_0.2.1_amd64.changes"), "r+") as f:
            contents = f.readlines()
            contents = contents[3:31]
            f.seek(0, 0)
            f.write("".join(contents))
            f.truncate()

        self.runCmd += self.tempSrcDir

    def check(self):
        try:
            super(IncludeRepo9Test, self).check()
        finally:
            shutil.rmtree(self.tempSrcDir)


class IncludeRepo10Test(BaseTest):
    """
    include packages to local repo: wrong signature + -ignore-signatures
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -ignore-signatures "

    def outputMatchPrepare(self, s):
        return gpgRemove(self, tempDirRemove(self, s))

    def prepare(self):
        super(IncludeRepo10Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()

        shutil.copytree(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"), os.path.join(self.tempSrcDir, "01"))

        with open(os.path.join(self.tempSrcDir, "01", "hardlink_0.2.1_amd64.changes"), "r+") as f:
            contents = f.read()
            f.seek(0, 0)
            f.write(contents.replace('Julian', 'Andrey'))
            f.truncate()

        self.runCmd += self.tempSrcDir

    def check(self):
        try:
            super(IncludeRepo10Test, self).check()
        finally:
            shutil.rmtree(self.tempSrcDir)


class IncludeRepo11Test(BaseTest):
    """
    include packages to local repo: unsigned + -accept-unsigned
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -accept-unsigned -keyring=${files}/aptly.pub "

    def outputMatchPrepare(self, s):
        return gpgRemove(self, tempDirRemove(self, s))

    def prepare(self):
        super(IncludeRepo11Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()

        shutil.copytree(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"), os.path.join(self.tempSrcDir, "01"))

        with open(os.path.join(self.tempSrcDir, "01", "hardlink_0.2.1_amd64.changes"), "r+") as f:
            contents = f.readlines()
            contents = contents[3:31]
            f.seek(0, 0)
            f.write("".join(contents))
            f.truncate()

        self.runCmd += self.tempSrcDir

    def check(self):
        try:
            super(IncludeRepo11Test, self).check()
        finally:
            shutil.rmtree(self.tempSrcDir)


class IncludeRepo12Test(BaseTest):
    """
    include packages to local repo: unsigned + -accept-unsigned + restriction breakage
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -accept-unsigned -keyring=${files}/aptly.pub "
    expectedCode = 1

    def outputMatchPrepare(self, s):
        return gpgRemove(self, tempDirRemove(self, s))

    def prepare(self):
        super(IncludeRepo12Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()

        shutil.copytree(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"), os.path.join(self.tempSrcDir, "01"))

        with open(os.path.join(self.tempSrcDir, "01", "hardlink_0.2.1_amd64.changes"), "r+") as f:
            contents = f.readlines()
            contents = contents[3:31]
            contents[3] = "Binary: hardlink-dbg\n"
            f.seek(0, 0)
            f.write("".join(contents))
            f.truncate()

        self.runCmd += self.tempSrcDir

    def check(self):
        try:
            super(IncludeRepo12Test, self).check()
        finally:
            shutil.rmtree(self.tempSrcDir)


class IncludeRepo13Test(BaseTest):
    """
    include packages to local repo: with denying uploaders.json
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -uploaders-file=${changes}/uploaders1.json -no-remove-files -keyring=${files}/aptly.pub ${changes}"
    expectedCode = 1

    def outputMatchPrepare(_, s):
        return changesRemove(_, gpgRemove(_, s))


class IncludeRepo14Test(BaseTest):
    """
    include packages to local repo: allow with uploaders.json
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -uploaders-file=${changes}/uploaders2.json -no-remove-files -keyring=${files}/aptly.pub ${changes}"

    def outputMatchPrepare(_, s):
        return changesRemove(_, gpgRemove(_, s))


class IncludeRepo15Test(BaseTest):
    """
    include packages to local repo: no uploaders.json
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -uploaders-file=${changes}/uploaders-404.json -no-remove-files -keyring=${files}/aptly.pub ${changes}"
    expectedCode = 1

    def outputMatchPrepare(_, s):
        return changesRemove(_, gpgRemove(_, s))


class IncludeRepo16Test(BaseTest):
    """
    include packages to local repo: malformed JSON
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -uploaders-file=${changes}/uploaders3.json -no-remove-files -keyring=${files}/aptly.pub ${changes}"
    expectedCode = 1

    def outputMatchPrepare(_, s):
        return changesRemove(_, gpgRemove(_, s))


class IncludeRepo17Test(BaseTest):
    """
    include packages to local repo: malformed rule
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -uploaders-file=${changes}/uploaders4.json -no-remove-files -keyring=${files}/aptly.pub ${changes}"
    expectedCode = 1

    def outputMatchPrepare(_, s):
        return changesRemove(_, gpgRemove(_, s))


class IncludeRepo18Test(BaseTest):
    """
    include packages to local repo: repo uploaders.json + global uploaders.json
    """
    fixtureCmds = [
        "aptly repo create -uploaders-file=${changes}/uploaders2.json unstable",
    ]
    runCmd = "aptly repo include -uploaders-file=${changes}/uploaders1.json -no-remove-files -keyring=${files}/aptly.pub ${changes}"

    def outputMatchPrepare(_, s):
        return changesRemove(_, gpgRemove(_, s))


class IncludeRepo19Test(BaseTest):
    """
    include packages to local repo: per-repo uploaders.json
    """
    fixtureCmds = [
        "aptly repo create -uploaders-file=${changes}/uploaders1.json unstable",
    ]
    runCmd = "aptly repo include -no-remove-files -keyring=${files}/aptly.pub ${changes}"
    expectedCode = 1

    def outputMatchPrepare(_, s):
        return changesRemove(_, gpgRemove(_, s))


class IncludeRepo20Test(BaseTest):
    """
    include packages to local repo: .changes file from directory (internal PGP implementation)
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -no-remove-files -keyring=${files}/aptly.pub ${changes}"
    outputMatchPrepare = gpgRemove
    configOverride = {"gpgProvider": "internal"}


class IncludeRepo21Test(BaseTest):
    """
    include packages to local repo: wrong signature (internal PGP implementation)
    """
    fixtureCmds = [
        "aptly repo create unstable",
    ]
    runCmd = "aptly repo include -keyring=${files}/aptly.pub "
    expectedCode = 1
    configOverride = {"gpgProvider": "internal"}

    def outputMatchPrepare(self, s):
        return gpgRemove(self, tempDirRemove(self, s))

    def prepare(self):
        super(IncludeRepo21Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()

        shutil.copytree(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"), os.path.join(self.tempSrcDir, "01"))

        with open(os.path.join(self.tempSrcDir, "01", "hardlink_0.2.1_amd64.changes"), "r+") as f:
            contents = f.read()
            f.seek(0, 0)
            f.write(contents.replace('Julian', 'Andrey'))
            f.truncate()

        self.runCmd += self.tempSrcDir

    def check(self):
        try:
            super(IncludeRepo21Test, self).check()
        finally:
            shutil.rmtree(self.tempSrcDir)


class IncludeRepo22Test(BaseTest):
    """
    include packages to local repo: missing files, but files aready in the pool
    """
    fixtureCmds = [
        "aptly repo create stable",
        "aptly repo create unstable",
        "aptly repo add stable ${changes}"
    ]
    runCmd = "aptly repo include -ignore-signatures -keyring=${files}/aptly.pub "

    def outputMatchPrepare(self, s):
        return gpgRemove(self, tempDirRemove(self, s))

    def prepare(self):
        super(IncludeRepo22Test, self).prepare()

        self.tempSrcDir = tempfile.mkdtemp()
        os.makedirs(os.path.join(self.tempSrcDir, "01"), 0o755)

        for path in ["hardlink_0.2.1.dsc", "hardlink_0.2.1_amd64.deb"]:
            shutil.copy(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes", path),
                        os.path.join(self.tempSrcDir, "01", path))

        path = "hardlink_0.2.1_amd64.changes"
        with open(os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes", path), "r") as source:
            with open(os.path.join(self.tempSrcDir, "01", path), "w") as dest:
                content = source.readlines()
                # remove reference to .tar.gz file
                content = [line for line in content if "hardlink_0.2.1.tar.gz" not in line]
                dest.write("".join(content))

        self.runCmd += self.tempSrcDir

    def check(self):
        try:
            super(IncludeRepo22Test, self).check()
        finally:
            shutil.rmtree(self.tempSrcDir)



import inspect
import os
import threading

from api_lib import TASK_SUCCEEDED, APITest

DefaultSigningOptions = {
    "Keyring": os.path.join(os.path.dirname(inspect.getsourcefile(APITest)), "files") + "/aptly.pub",
}


class PublishAPITestRepo(APITest):
    """
    POST /publish/:prefix (local repos), GET /publish
    """
    fixtureGpg = True

    def check(self):
        repo_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb", "pyspi_0.6.1-1.3.dsc",
                                     "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                                     "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)

        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        # publishing under prefix, default distribution
        prefix = self.random_name()
        task = self.post_task(
            "/api/publish/" + prefix,
            json={
                 "SourceKind": "local",
                 "Sources": [{"Name": repo_name}],
                 "Signing": DefaultSigningOptions,
            }
        )
        self.check_task(task)
        repo_expected = {
            'AcquireByHash': False,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'wheezy',
            'Label': '',
            'Origin': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Path': prefix + '/' + 'wheezy',
            'Prefix': prefix,
            'SkipContents': False,
            'MultiDist': False,
            'SourceKind': 'local',
            'Sources': [{'Component': 'main', 'Name': repo_name}],
            'Storage': '',
            'Suite': ''}

        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        self.check_exists("public/" + prefix + "/dists/wheezy/Release")
        self.check_exists("public/" + prefix +
                          "/dists/wheezy/main/binary-i386/Packages")
        self.check_exists("public/" + prefix +
                          "/dists/wheezy/main/Contents-i386.gz")
        self.check_exists("public/" + prefix +
                          "/dists/wheezy/main/source/Sources")
        self.check_exists(
            "public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")

        # publishing under root, custom distribution, architectures
        distribution = self.random_name()
        task = self.post_task(
            "/api/publish/:.",
            json={
                 "SourceKind": "local",
                 "Sources": [{"Name": repo_name}],
                 "Signing": DefaultSigningOptions,
                 "Distribution": distribution,
                 "Architectures": ["i386", "amd64"],
            }
        )
        self.check_task(task)
        repo2_expected = {
            'AcquireByHash': False,
            'Architectures': ['amd64', 'i386'],
            'Codename': '',
            'Distribution': distribution,
            'Label': '',
            'Origin': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Path': './' + distribution,
            'Prefix': ".",
            'SkipContents': False,
            'MultiDist': False,
            'SourceKind': 'local',
            'Sources': [{'Component': 'main', 'Name': repo_name}],
            'Storage': '',
            'Suite': ''}
        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        self.check_exists("public/dists/" + distribution + "/Release")
        self.check_exists("public/dists/" + distribution +
                          "/main/binary-i386/Packages")
        self.check_exists("public/dists/" + distribution +
                          "/main/Contents-i386.gz")
        self.check_exists("public/dists/" + distribution +
                          "/main/binary-amd64/Packages")
        self.check_not_exists(
            "public/dists/" + distribution + "/main/Contents-amd64.gz")
        self.check_exists(
            "public/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")

        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())
        self.check_in(repo2_expected, all_repos.json())


class PublishAPITestRepoMultiDist(APITest):
    """
    Test MultiDist publishing to subdirectory
    """
    fixtureGpg = True

    def check(self):
        repo_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo_name, "DefaultDistribution": "bookworm"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb", "pyspi_0.6.1-1.3.dsc",
                                     "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                                     "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)

        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        # publishing under prefix, default distribution
        prefix = self.random_name()
        task = self.post_task(
            "/api/publish/" + prefix,
            json={
                 "SourceKind": "local",
                 "MultiDist": True,
                 "Sources": [{"Name": repo_name}],
                 "Signing": DefaultSigningOptions,
            }
        )
        self.check_task(task)
        repo_expected = {
            'AcquireByHash': False,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'bookworm',
            'Label': '',
            'Origin': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Path': prefix + '/' + 'bookworm',
            'Prefix': prefix,
            'SkipContents': False,
            'MultiDist': True,
            'SourceKind': 'local',
            'Sources': [{'Component': 'main', 'Name': repo_name}],
            'Storage': '',
            'Suite': ''}

        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        self.check_exists("public/" + prefix + "/dists/bookworm/Release")
        self.check_exists("public/" + prefix +
                          "/dists/bookworm/main/binary-i386/Packages")
        self.check_exists("public/" + prefix +
                          "/dists/bookworm/main/Contents-i386.gz")
        self.check_exists("public/" + prefix +
                          "/dists/bookworm/main/source/Sources")
        self.check_exists(
            "public/" + prefix + "/pool/bookworm/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")


class PublishSnapshotAPITest(APITest):
    """
    POST /publish/:prefix (snapshots), GET /publish
    """

    def check(self):
        repo_name = self.random_name()
        snapshot_name = self.random_name()
        self.check_equal(
            self.post("/api/repos", json={"Name": repo_name}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        task = self.post_task("/api/repos/" + repo_name + '/snapshots', json={'Name': snapshot_name})
        self.check_task(task)

        prefix = self.random_name()
        task = self.post_task(
            "/api/publish/" + prefix,
            json={
                "AcquireByHash": True,
                "SourceKind": "snapshot",
                "Sources": [{"Name": snapshot_name}],
                "Signing": DefaultSigningOptions,
                "Distribution": "squeeze",
                "NotAutomatic": "yes",
                "ButAutomaticUpgrades": "yes",
                "Origin": "earth",
                "Label": "fun",
            }
        )
        self.check_task(task)

        _id = task.json()['ID']
        resp = self.get("/api/tasks/" + str(_id) + "/detail")
        self.check_equal(resp.json()['RemainingNumberOfPackages'], 0)
        self.check_equal(resp.json()['TotalNumberOfPackages'], 1)

        repo_expected = {
            'AcquireByHash': True,
            'Architectures': ['i386'],
            'Codename': '',
            'Distribution': 'squeeze',
            'Label': 'fun',
            'Origin': 'earth',
            'MultiDist': False,
            'NotAutomatic': 'yes',
            'ButAutomaticUpgrades': 'yes',
            'Path': prefix + '/' + 'squeeze',
            'Prefix': prefix,
            'SkipContents': False,
            'SourceKind': 'snapshot',
            'Sources': [{'Component': 'main', 'Name': snapshot_name}],
            'Storage': '',
            'Suite': '',
        }
        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        self.check_exists("public/" + prefix + "/dists/squeeze/Release")
        self.check_exists("public/" + prefix +
                          "/dists/squeeze/main/binary-i386/by-hash")
        self.check_exists("public/" + prefix +
                          "/dists/squeeze/main/binary-i386/Packages")
        self.check_exists("public/" + prefix +
                          "/dists/squeeze/main/Contents-i386.gz")
        self.check_exists(
            "public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")


class PublishUpdateAPITestRepo(APITest):
    """
    PUT /publish/:prefix/:distribution (local repos), DELETE /publish/:prefix/:distribution
    """
    fixtureGpg = True

    def check(self):
        repo_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(
            self.upload("/api/files/" + d,
                        "pyspi_0.6.1-1.3.dsc",
                        "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                        "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        prefix = self.random_name()
        task = self.post_task(
            "/api/publish/" + prefix,
            json={
                "Architectures": ["i386", "source"],
                "SourceKind": "local",
                "Sources": [{"Name": repo_name}],
                "Signing": DefaultSigningOptions,
            }
        )
        self.check_task(task)

        self.check_not_exists(
            "public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_exists("public/" + prefix +
                          "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                         "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        task = self.delete_task("/api/repos/" + repo_name + "/packages/",
                                json={"PackageRefs": ['Psource pyspi 0.6.1-1.4 f8f1daa806004e89']})
        self.check_task(task)

        # Update and switch AcquireByHash on.
        task = self.put_task(
            "/api/publish/" + prefix + "/wheezy",
            json={
                "AcquireByHash": True,
                "Signing": DefaultSigningOptions,
            }
        )
        self.check_task(task)
        repo_expected = {
            'AcquireByHash': True,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'wheezy',
            'Label': '',
            'Origin': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Path': prefix + '/' + 'wheezy',
            'Prefix': prefix,
            'SkipContents': False,
            'MultiDist': False,
            'SourceKind': 'local',
            'Sources': [{'Component': 'main', 'Name': repo_name}],
            'Storage': '',
            'Suite': ''}

        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        self.check_exists("public/" + prefix +
                          "/dists/wheezy/main/binary-i386/by-hash")

        self.check_exists(
            "public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_not_exists(
            "public/" + prefix + "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")

        task = self.delete_task("/api/publish/" + prefix + "/wheezy")
        self.check_task(task)
        self.check_not_exists("public/" + prefix + "dists/")


class PublishUpdateAPIMultiDist(APITest):
    """
    Test MultiDist publishing to subdirectory
    """
    fixtureGpg = True

    def check(self):
        repo_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo_name, "DefaultDistribution": "bookworm"}).status_code, 201)

        d = self.random_name()
        self.check_equal(
            self.upload("/api/files/" + d,
                        "pyspi_0.6.1-1.3.dsc",
                        "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                        "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        prefix = self.random_name()
        task = self.post_task(
            "/api/publish/" + prefix,
            json={
                "Architectures": ["i386", "source"],
                "SourceKind": "local",
                "Sources": [{"Name": repo_name}],
                "Signing": DefaultSigningOptions,
            }
        )
        self.check_task(task)

        self.check_not_exists(
            "public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_exists("public/" + prefix +
                          "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                         "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        task = self.delete_task("/api/repos/" + repo_name + "/packages/",
                                json={"PackageRefs": ['Psource pyspi 0.6.1-1.4 f8f1daa806004e89']})
        self.check_task(task)

        # Update and switch MultiDist on.
        task = self.put_task(
            "/api/publish/" + prefix + "/bookworm",
            json={
                "MultiDist": True,
                "Signing": DefaultSigningOptions,
            }
        )
        self.check_task(task)
        repo_expected = {
            'AcquireByHash': False,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'bookworm',
            'Label': '',
            'Origin': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Path': prefix + '/' + 'bookworm',
            'Prefix': prefix,
            'SkipContents': False,
            'MultiDist': True,
            'SourceKind': 'local',
            'Sources': [{'Component': 'main', 'Name': repo_name}],
            'Storage': '',
            'Suite': ''}

        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        self.check_exists(
            "public/" + prefix + "/pool/bookworm/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_not_exists(
            "public/" + prefix + "/pool/bookworm/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")

        task = self.delete_task("/api/publish/" + prefix + "/bookworm")
        self.check_task(task)
        self.check_not_exists("public/" + prefix + "dists/")


class PublishConcurrentUpdateAPITestRepo(APITest):
    """
    PUT /publish/:prefix/:distribution (local repos), DELETE /publish/:prefix/:distribution
    """
    fixtureGpg = True

    def check(self):
        repo_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(
            self.upload("/api/files/" + d,
                        "pyspi_0.6.1-1.3.dsc",
                        "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                        "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        prefix = self.random_name()
        task = self.post_task(
            "/api/publish/" + prefix,
            json={
                "Architectures": ["i386", "source"],
                "SourceKind": "local",
                "Sources": [{"Name": repo_name}],
                "Signing": DefaultSigningOptions,
            }
        )
        self.check_task(task)

        self.check_not_exists(
            "public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_exists("public/" + prefix +
                          "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                         "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        task = self.delete_task("/api/repos/" + repo_name + "/packages/",
                                json={"PackageRefs": ['Psource pyspi 0.6.1-1.4 f8f1daa806004e89']})
        self.check_task(task)

        def _do_update(result, index):
            resp = self.put_task(
                "/api/publish/" + prefix + "/wheezy",
                json={
                    "AcquireByHash": True,
                    "Signing": DefaultSigningOptions,
                }
            )
            try:
                self.check_equal(resp.json()['State'], TASK_SUCCEEDED)
            except BaseException as e:
                result[index] = e

        n_workers = 10
        worker_results = [None] * n_workers
        tasks = [threading.Thread(target=_do_update, args=(worker_results, i,)) for i in range(n_workers)]
        [task.start() for task in tasks]
        [task.join() for task in tasks]
        for result in worker_results:
            if isinstance(result, BaseException):
                raise result

        repo_expected = {
            'AcquireByHash': True,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'wheezy',
            'Label': '',
            'Origin': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Path': prefix + '/' + 'wheezy',
            'Prefix': prefix,
            'SkipContents': False,
            'MultiDist': False,
            'SourceKind': 'local',
            'Sources': [{'Component': 'main', 'Name': repo_name}],
            'Storage': '',
            'Suite': ''}

        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        self.check_exists("public/" + prefix +
                          "/dists/wheezy/main/binary-i386/by-hash")

        self.check_exists(
            "public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_not_exists(
            "public/" + prefix + "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")

        task = self.delete_task("/api/publish/" + prefix + "/wheezy")
        self.check_task(task)
        self.check_not_exists("public/" + prefix + "dists/")


class PublishUpdateSkipCleanupAPITestRepo(APITest):
    """
    PUT /publish/:prefix/:distribution (local repos), DELETE /publish/:prefix/:distribution
    """
    fixtureGpg = True

    def check(self):
        repo_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(
            self.upload("/api/files/" + d,
                        "pyspi_0.6.1-1.3.dsc",
                        "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                        "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        prefix = self.random_name()
        task = self.post_task("/api/publish/" + prefix,
                              json={
                                  "Architectures": ["i386", "source"],
                                  "SourceKind": "local",
                                  "Sources": [{"Name": repo_name}],
                                  "Signing": DefaultSigningOptions,
                              })
        self.check_task(task)

        self.check_not_exists(
            "public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_exists("public/" + prefix +
                          "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")

        # Publish two repos, so that deleting one while skipping cleanup will
        # not delete the whole prefix.
        task = self.post_task("/api/publish/" + prefix,
                              json={
                                  "Architectures": ["i386", "source"],
                                  "Distribution": "otherdist",
                                  "SourceKind": "local",
                                  "Sources": [{"Name": repo_name}],
                                  "Signing": DefaultSigningOptions,
                              })
        self.check_task(task)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                         "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        task = self.delete_task("/api/repos/" + repo_name + "/packages/",
                                json={"PackageRefs": ['Psource pyspi 0.6.1-1.4 f8f1daa806004e89']})
        self.check_task(task)

        task = self.put_task("/api/publish/" + prefix + "/wheezy",
                             json={
                                 "Signing": DefaultSigningOptions,
                                 "SkipCleanup": True,
                             })
        self.check_task(task)
        repo_expected = {
            'AcquireByHash': False,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'wheezy',
            'Label': '',
            'Origin': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Path': prefix + '/' + 'wheezy',
            'Prefix': prefix,
            'SkipContents': False,
            'MultiDist': False,
            'SourceKind': 'local',
            'Sources': [{'Component': 'main', 'Name': repo_name}],
            'Storage': '',
            'Suite': ''}

        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        self.check_exists(
            "public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_exists("public/" + prefix +
                          "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")

        task = self.delete_task("/api/publish/" + prefix + "/wheezy", params={"SkipCleanup": "1"})
        self.check_task(task)
        self.check_exists("public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_exists("public/" + prefix + "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")


class PublishSwitchAPITestRepo(APITest):
    """
    PUT /publish/:prefix/:distribution (snapshots), DELETE /publish/:prefix/:distribution
    """
    fixtureGpg = True

    def check(self):
        repo_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(
            self.upload("/api/files/" + d,
                        "pyspi_0.6.1-1.3.dsc",
                        "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                        "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        snapshot1_name = self.random_name()
        task = self.post_task("/api/repos/" + repo_name + '/snapshots', json={'Name': snapshot1_name})
        self.check_task(task)

        prefix = self.random_name()
        task = self.post_task(
            "/api/publish/" + prefix,
            json={
                "Architectures": ["i386", "source"],
                "SourceKind": "snapshot",
                "Sources": [{"Name": snapshot1_name}],
                "Signing": DefaultSigningOptions,
            })
        self.check_task(task)

        repo_expected = {
            'AcquireByHash': False,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'wheezy',
            'Label': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Origin': '',
            'Path': prefix + '/' + 'wheezy',
            'Prefix': prefix,
            'SkipContents': False,
            'MultiDist': False,
            'SourceKind': 'snapshot',
            'Sources': [{'Component': 'main', 'Name': snapshot1_name}],
            'Storage': '',
            'Suite': ''}
        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        self.check_not_exists(
            "public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_exists("public/" + prefix +
                          "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                         "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        task = self.delete_task("/api/repos/" + repo_name + "/packages/",
                                json={"PackageRefs": ['Psource pyspi 0.6.1-1.4 f8f1daa806004e89']})
        self.check_task(task)

        snapshot2_name = self.random_name()
        task = self.post_task("/api/repos/" + repo_name + '/snapshots', json={'Name': snapshot2_name})
        self.check_task(task)

        task = self.put_task(
            "/api/publish/" + prefix + "/wheezy",
            json={
                "Snapshots": [{"Component": "main", "Name": snapshot2_name}],
                "Signing": DefaultSigningOptions,
                "SkipContents": True,
            })
        self.check_task(task)
        repo_expected = {
            'AcquireByHash': False,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'wheezy',
            'Label': '',
            'Origin': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Path': prefix + '/' + 'wheezy',
            'Prefix': prefix,
            'SkipContents': True,
            'MultiDist': False,
            'SourceKind': 'snapshot',
            'Sources': [{'Component': 'main', 'Name': snapshot2_name}],
            'Storage': '',
            'Suite': ''}

        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        self.check_exists(
            "public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_not_exists(
            "public/" + prefix + "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")

        task = self.delete_task("/api/publish/" + prefix + "/wheezy")
        self.check_task(task)
        self.check_not_exists("public/" + prefix + "dists/")


class PublishSwitchAPISkipCleanupTestRepo(APITest):
    """
    PUT /publish/:prefix/:distribution (snapshots), DELETE /publish/:prefix/:distribution
    """
    fixtureGpg = True

    def check(self):
        repo_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(
            self.upload("/api/files/" + d,
                        "pyspi_0.6.1-1.3.dsc",
                        "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                        "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        snapshot1_name = self.random_name()
        task = self.post_task("/api/repos/" + repo_name + '/snapshots', json={'Name': snapshot1_name})
        self.check_task(task)

        prefix = self.random_name()
        task = self.post_task("/api/publish/" + prefix,
                              json={
                                  "Architectures": ["i386", "source"],
                                  "SourceKind": "snapshot",
                                  "Sources": [{"Name": snapshot1_name}],
                                  "Signing": DefaultSigningOptions,
                              })

        self.check_task(task)
        repo_expected = {
            'AcquireByHash': False,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'wheezy',
            'Label': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Origin': '',
            'Path': prefix + '/' + 'wheezy',
            'Prefix': prefix,
            'SkipContents': False,
            'MultiDist': False,
            'SourceKind': 'snapshot',
            'Sources': [{'Component': 'main', 'Name': snapshot1_name}],
            'Storage': '',
            'Suite': ''}
        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        self.check_not_exists(
            "public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_exists("public/" + prefix +
                          "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")

        # Publish two snapshots, so that deleting one while skipping cleanup will
        # not delete the whole prefix.
        task = self.post_task("/api/publish/" + prefix,
                              json={
                                  "Architectures": ["i386", "source"],
                                  "Distribution": "otherdist",
                                  "SourceKind": "snapshot",
                                  "Sources": [{"Name": snapshot1_name}],
                                  "Signing": DefaultSigningOptions,
                              })

        self.check_task(task)
        repo_expected = {
            'AcquireByHash': False,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'otherdist',
            'Label': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Origin': '',
            'Path': prefix + '/' + 'otherdist',
            'Prefix': prefix,
            'SkipContents': False,
            'MultiDist': False,
            'SourceKind': 'snapshot',
            'Sources': [{'Component': 'main', 'Name': snapshot1_name}],
            'Storage': '',
            'Suite': ''}
        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                         "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)
        task = self.post_task("/api/repos/" + repo_name + "/file/" + d)
        self.check_task(task)

        task = self.delete_task("/api/repos/" + repo_name + "/packages/",
                                json={"PackageRefs": ['Psource pyspi 0.6.1-1.4 f8f1daa806004e89']})
        self.check_task(task)

        snapshot2_name = self.random_name()
        task = self.post_task("/api/repos/" + repo_name + '/snapshots', json={'Name': snapshot2_name})
        self.check_task(task)

        task = self.put_task("/api/publish/" + prefix + "/wheezy",
                             json={
                                 "Snapshots": [{"Component": "main", "Name": snapshot2_name}],
                                 "Signing": DefaultSigningOptions,
                                 "SkipCleanup": True,
                                 "SkipContents": True,
                             })
        self.check_task(task)
        repo_expected = {
            'AcquireByHash': False,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'wheezy',
            'Label': '',
            'Origin': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Path': prefix + '/' + 'wheezy',
            'Prefix': prefix,
            'SkipContents': True,
            'MultiDist': False,
            'SourceKind': 'snapshot',
            'Sources': [{'Component': 'main', 'Name': snapshot2_name}],
            'Storage': '',
            'Suite': ''}

        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())

        self.check_exists("public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_exists("public/" + prefix + "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")

        task = self.delete_task("/api/publish/" + prefix + "/wheezy", params={"SkipCleanup": "1"})
        self.check_task(task)
        self.check_exists("public/" + prefix + "/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb")
        self.check_exists("public/" + prefix + "/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc")


class PublishShowAPITestRepo(APITest):
    """
    GET /publish/:prefix/:distribution
    """

    def check(self):
        repo1_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo1_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(
            self.upload("/api/files/" + d,
                        "pyspi_0.6.1-1.3.dsc",
                        "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                        "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)
        self.check_equal(self.post_task("/api/repos/" + repo1_name + "/file/" + d).status_code, 200)

        # publishing under prefix, default distribution
        prefix = self.random_name()
        self.check_equal(self.post(
            "/api/publish/" + prefix,
            json={
                 "Architectures": ["i386", "source"],
                 "SourceKind": "local",
                 "Sources": [{"Component": "main", "Name": repo1_name}],
                 "Signing": DefaultSigningOptions,
            }
        ).status_code, 201)

        repo_expected = {
            'AcquireByHash': False,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'wheezy',
            'Label': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Origin': '',
            'Path': prefix + '/' + 'wheezy',
            'Prefix': prefix,
            'SkipContents': False,
            'MultiDist': False,
            'SourceKind': 'local',
            'Sources': [{'Component': 'main', 'Name': repo1_name}],
            'Storage': '',
            'Suite': ''}
        repo = self.get("/api/publish/" + prefix + "/wheezy")
        self.check_equal(repo.status_code, 200)
        self.check_equal(repo_expected, repo.json())


class ServePublishedListTestRepo(APITest):
    """
    GET /repos
    """

    def check(self):
        d = "libboost-program-options-dev_1.62.0.1"
        r = "bar"
        f = "libboost-program-options-dev_1.62.0.1_i386.deb"

        self.check_equal(self.upload("/api/files/" + d, f).status_code, 200)

        self.check_equal(self.post("/api/repos", json={
            "Name": r,
            "Comment": "test repo",
            "DefaultDistribution": r,
            "DefaultComponent": "main"
        }).status_code, 201)

        self.check_equal(self.post(f"/api/repos/{r}/file/{d}").status_code, 200)

        self.check_equal(self.post("/api/publish/filesystem:apiandserve:", json={
            "SourceKind": "local",
            "Sources": [
                {
                    "Component": "main",
                    "Name": r
                }
            ],
            "Distribution": r,
            "Signing":  {
                "Skip": True
            }
        }).status_code, 201)

        get = self.get("/repos")
        expected_content_type = "text/html; charset=utf-8"
        if get.headers['content-type'] != expected_content_type:
            raise Exception(f"Received content-type {get.headers['content-type']} was not: {expected_content_type}")

        excepted_content = b'<pre>\n<a href="apiandserve/">apiandserve</a>\n</pre>'
        if excepted_content != get.content:
            raise Exception(f"Expected content {excepted_content} was not: {get.content}")


class ServePublishedTestRepo(APITest):
    """
    GET /repos/:storage/*pkgPath
    """

    def check(self):
        d = self.random_name()
        r = self.random_name()
        f = "libboost-program-options-dev_1.62.0.1_i386.deb"

        self.check_equal(self.upload("/api/files/" + d, f).status_code, 200)

        self.check_equal(self.post("/api/repos", json={
            "Name": r,
            "Comment": "test repo",
            "DefaultDistribution": r,
            "DefaultComponent": "main"
        }).status_code, 201)

        self.check_equal(self.post(f"/api/repos/{r}/file/{d}").status_code, 200)

        self.check_equal(self.post("/api/publish/filesystem:apiandserve:", json={
            "SourceKind": "local",
            "Sources": [
                {
                    "Component": "main",
                    "Name": r
                }
            ],
            "Distribution": r,
            "Signing":  {
                "Skip": True
            }
        }).status_code, 201)

        get = self.get(f"/repos/apiandserve/pool/main/b/boost-defaults/{f}")
        deb_content_types = [
            "application/x-deb",
            "application/x-debian-package",
            "application/vnd.debian.binary-package"
        ]
        if get.headers['content-type'] not in deb_content_types:
            raise Exception(f"Received content-type {get.headers['content-type']} not one of expected: {deb_content_types}")

        if len(get.content) != 3428:
            raise Exception(f"Expected file size 3428 bytes != {len(get.content)} bytes")


class ServePublishedNotFoundTestRepo(APITest):
    """
    GET /repos/:storage/*pkgPath
    """

    def check(self):
        d = self.random_name()
        r = self.random_name()
        f = "libboost-program-options-dev_1.62.0.1_i386.deb"

        self.check_equal(self.upload("/api/files/" + d, f).status_code, 200)

        self.check_equal(self.post("/api/repos", json={
            "Name": r,
            "Comment": "test repo",
            "DefaultDistribution": r,
            "DefaultComponent": "main"
        }).status_code, 201)

        self.check_equal(self.post(f"/api/repos/{r}/file/{d}").status_code, 200)

        self.check_equal(self.post("/api/publish/filesystem:apiandserve:", json={
            "SourceKind": "local",
            "Sources": [
                {
                    "Component": "main",
                    "Name": r
                }
            ],
            "Distribution": r,
            "Signing":  {
                "Skip": True
            }
        }).status_code, 201)

        get = self.get("/repos/apiandserve/pool/main/b/boost-defaults/i-dont-exist")
        if get.status_code != 404:
            raise Exception(f"Expected status 404 != {get.status_code}")


class PublishSourcesAddAPITestRepo(APITest):
    """
    POST /publish/:prefix/:distribution/sources
    """
    fixtureGpg = True

    def check(self):
        repo1_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo1_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb", "pyspi_0.6.1-1.3.dsc",
                                     "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                                     "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo1_name + "/file/" + d).status_code, 200)

        # publishing under prefix, default distribution
        prefix = self.random_name()
        self.check_equal(self.post(
            "/api/publish/" + prefix,
            json={
                 "SourceKind": "local",
                 "Sources": [{"Component": "main", "Name": repo1_name}],
                 "Signing": DefaultSigningOptions,
            }
        ).status_code, 201)

        repo2_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo2_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo2_name + "/file/" + d).status_code, 200)

        # Actual test
        self.check_equal(self.post(
            "/api/publish/" + prefix + "/wheezy/sources",
            json={"Component": "test", "Name": repo2_name}
        ).status_code, 201)

        sources_expected = [{"Component": "main", "Name": repo1_name}, {"Component": "test", "Name": repo2_name}]
        sources = self.get("/api/publish/" + prefix + "/wheezy/sources")
        self.check_equal(sources.status_code, 200)
        self.check_equal(sources_expected, sources.json())


class PublishSourceUpdateAPITestRepo(APITest):
    """
    PUT /publish/:prefix/:distribution/sources/main
    """
    fixtureGpg = True

    def check(self):
        repo1_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo1_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb", "pyspi_0.6.1-1.3.dsc",
                                     "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                                     "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo1_name + "/file/" + d).status_code, 200)

        # publishing under prefix, default distribution
        prefix = self.random_name()
        self.check_equal(self.post(
            "/api/publish/" + prefix,
            json={
                 "SourceKind": "local",
                 "Sources": [{"Component": "main", "Name": repo1_name}],
                 "Signing": DefaultSigningOptions,
            }
        ).status_code, 201)

        repo2_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo2_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo2_name + "/file/" + d).status_code, 200)

        # Actual test
        self.check_equal(self.put(
            "/api/publish/" + prefix + "/wheezy/sources/main",
            json={"Component": "main", "Name": repo2_name}
        ).status_code, 200)

        sources_expected = [{"Component": "main", "Name": repo2_name}]
        sources = self.get("/api/publish/" + prefix + "/wheezy/sources")
        self.check_equal(sources.status_code, 200)
        self.check_equal(sources_expected, sources.json())


class PublishSourcesUpdateAPITestRepo(APITest):
    """
    PUT /publish/:prefix/:distribution/sources
    """
    fixtureGpg = True

    def check(self):
        repo1_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo1_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb", "pyspi_0.6.1-1.3.dsc",
                                     "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                                     "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo1_name + "/file/" + d).status_code, 200)

        repo2_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo2_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo2_name + "/file/" + d).status_code, 200)

        # publishing under prefix, default distribution
        prefix = self.random_name()
        self.check_equal(self.post(
            "/api/publish/" + prefix,
            json={
                 "SourceKind": "local",
                 "Sources": [{"Component": "main", "Name": repo1_name}],
                 "Signing": DefaultSigningOptions,
            }
        ).status_code, 201)

        # Actual test
        self.check_equal(self.put(
            "/api/publish/" + prefix + "/wheezy/sources",
            json=[{"Component": "test", "Name": repo1_name}, {"Component": "other-test", "Name": repo2_name}]
        ).status_code, 200)

        sources_expected = [{"Component": "other-test", "Name": repo2_name}, {"Component": "test", "Name": repo1_name}]
        sources = self.get("/api/publish/" + prefix + "/wheezy/sources")
        self.check_equal(sources.status_code, 200)
        self.check_equal(sources_expected, sources.json())


class PublishSourceRemoveAPITestRepo(APITest):
    """
    DELETE /publish/:prefix/:distribution/sources/test
    """
    fixtureGpg = True

    def check(self):
        repo1_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo1_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb", "pyspi_0.6.1-1.3.dsc",
                                     "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                                     "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo1_name + "/file/" + d).status_code, 200)

        repo2_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo2_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo2_name + "/file/" + d).status_code, 200)

        # publishing under prefix, default distribution
        prefix = self.random_name()
        self.check_equal(self.post(
            "/api/publish/" + prefix,
            json={
                 "SourceKind": "local",
                 "Sources": [{"Component": "main", "Name": repo1_name}, {"Component": "test", "Name": repo2_name}],
                 "Signing": DefaultSigningOptions,
            }
        ).status_code, 201)

        # Actual test
        self.check_equal(self.delete("/api/publish/" + prefix + "/wheezy/sources/test").status_code, 200)

        sources_expected = [{"Component": "main", "Name": repo1_name}]
        sources = self.get("/api/publish/" + prefix + "/wheezy/sources")
        self.check_equal(sources.status_code, 200)
        self.check_equal(sources_expected, sources.json())


class PublishSourcesDropAPITestRepo(APITest):
    """
    DELETE /publish/:prefix/:distribution/sources
    """
    fixtureGpg = True

    def check(self):
        repo1_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo1_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb", "pyspi_0.6.1-1.3.dsc",
                                     "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                                     "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo1_name + "/file/" + d).status_code, 200)

        repo2_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo2_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo2_name + "/file/" + d).status_code, 200)

        # publishing under prefix, default distribution
        prefix = self.random_name()
        self.check_equal(self.post(
            "/api/publish/" + prefix,
            json={
                 "SourceKind": "local",
                 "Sources": [{"Component": "main", "Name": repo1_name}, {"Component": "test", "Name": repo2_name}],
                 "Signing": DefaultSigningOptions,
            }
        ).status_code, 201)

        self.check_equal(self.delete("/api/publish/" + prefix + "/wheezy/sources/test").status_code, 200)

        # Actual test
        self.check_equal(self.delete("/api/publish/" + prefix + "/wheezy/sources").status_code, 200)

        self.check_equal(self.get("/api/publish/" + prefix + "/wheezy/sources").status_code, 404)


class PublishSourcesListAPITestRepo(APITest):
    """
    GET /publish/:prefix/:distribution/sources
    """
    fixtureGpg = True

    def check(self):
        repo1_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo1_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb", "pyspi_0.6.1-1.3.dsc",
                                     "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                                     "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo1_name + "/file/" + d).status_code, 200)

        repo2_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo2_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo2_name + "/file/" + d).status_code, 200)

        # publishing under prefix, default distribution
        prefix = self.random_name()
        self.check_equal(self.post(
            "/api/publish/" + prefix,
            json={
                 "SourceKind": "local",
                 "Sources": [{"Component": "main", "Name": repo1_name}],
                 "Signing": DefaultSigningOptions,
            }
        ).status_code, 201)

        # Actual test
        self.check_equal(self.post(
            "/api/publish/" + prefix + "/wheezy/sources",
            json={"Component": "test", "Name": repo1_name}
        ).status_code, 201)

        self.check_equal(self.put(
            "/api/publish/" + prefix + "/wheezy/sources/main",
            json={"Component": "main", "Name": repo2_name}
        ).status_code, 200)

        self.check_equal(self.delete("/api/publish/" + prefix + "/wheezy/sources/main").status_code, 200)

        sources_expected = [{"Component": "test", "Name": repo1_name}]
        sources = self.get("/api/publish/" + prefix + "/wheezy/sources")
        self.check_equal(sources.status_code, 200)
        self.check_equal(sources_expected, sources.json())


class PublishSourceReplaceAPITestRepo(APITest):
    """
    PUT /publish/:prefix/:distribution/sources/main
    """
    fixtureGpg = True

    def check(self):
        repo1_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo1_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb", "pyspi_0.6.1-1.3.dsc",
                                     "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                                     "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo1_name + "/file/" + d).status_code, 200)

        repo2_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo2_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo2_name + "/file/" + d).status_code, 200)

        # publishing under prefix, default distribution
        prefix = self.random_name()
        self.check_equal(self.post(
            "/api/publish/" + prefix,
            json={
                 "SourceKind": "local",
                 "Sources": [{"Component": "main", "Name": repo1_name}],
                 "Signing": DefaultSigningOptions,
            }
        ).status_code, 201)

        # Actual test
        self.check_equal(self.put(
            "/api/publish/" + prefix + "/wheezy/sources/main",
            json={"Component": "test", "Name": repo2_name}
        ).status_code, 200)

        sources_expected = [{"Component": "test", "Name": repo2_name}]
        sources = self.get("/api/publish/" + prefix + "/wheezy/sources")
        self.check_equal(sources.status_code, 200)
        self.check_equal(sources_expected, sources.json())


class PublishUpdateSourcesAPITestRepo(APITest):
    """
    POST /publish/:prefix/:distribution/update
    """
    fixtureGpg = True

    def check(self):
        repo1_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo1_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb", "pyspi_0.6.1-1.3.dsc",
                                     "pyspi_0.6.1-1.3.diff.gz", "pyspi_0.6.1.orig.tar.gz",
                                     "pyspi-0.6.1-1.3.stripped.dsc").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo1_name + "/file/" + d).status_code, 200)

        repo2_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo2_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo2_name + "/file/" + d).status_code, 200)

        repo3_name = self.random_name()
        self.check_equal(self.post(
            "/api/repos", json={"Name": repo3_name, "DefaultDistribution": "wheezy"}).status_code, 201)

        d = self.random_name()
        self.check_equal(self.upload("/api/files/" + d,
                                     "libboost-program-options-dev_1.49.0.1_i386.deb").status_code, 200)

        self.check_equal(self.post("/api/repos/" + repo3_name + "/file/" + d).status_code, 200)

        # publishing under prefix, default distribution
        prefix = self.random_name()
        self.check_equal(self.post(
            "/api/publish/" + prefix,
            json={
                 "Signing": DefaultSigningOptions,
                 "SourceKind": "local",
                 "Sources": [{"Component": "main", "Name": repo1_name}, {"Component": "test", "Name": repo2_name}],
            }
        ).status_code, 201)

        # remove 'main' component
        self.check_equal(self.delete("/api/publish/" + prefix + "/wheezy/sources/main").status_code, 200)

        # update 'test' component
        self.check_equal(self.put(
            "/api/publish/" + prefix + "/wheezy/sources/test",
            json={"Component": "test", "Name": repo1_name}
        ).status_code, 200)

        # add 'other-test' component
        self.check_equal(self.post(
            "/api/publish/" + prefix + "/wheezy/sources",
            json={"Component": "other-test", "Name": repo3_name}
        ).status_code, 201)

        sources_expected = [{"Component": "other-test", "Name": repo3_name}, {"Component": "test", "Name": repo1_name}]
        sources = self.get("/api/publish/" + prefix + "/wheezy/sources")
        self.check_equal(sources.status_code, 200)
        self.check_equal(sources_expected, sources.json())

        # update published repository and publish new content
        self.check_equal(self.post(
            "/api/publish/" + prefix + "/wheezy/update",
            json={
                "AcquireByHash": True,
                "MultiDist": False,
                "Signing": DefaultSigningOptions,
                "SkipBz2": True,
                "SkipContents": True,
            }
        ).status_code, 200)

        repo_expected = {
            'AcquireByHash': True,
            'Architectures': ['i386', 'source'],
            'Codename': '',
            'Distribution': 'wheezy',
            'Label': '',
            'Origin': '',
            'NotAutomatic': '',
            'ButAutomaticUpgrades': '',
            'Path': prefix + '/' + 'wheezy',
            'Prefix': prefix,
            'SkipContents': True,
            'MultiDist': False,
            'SourceKind': 'local',
            'Sources': [{"Component": "other-test", "Name": repo3_name}, {"Component": "test", "Name": repo1_name}],
            'Storage': '',
            'Suite': ''}

        all_repos = self.get("/api/publish")
        self.check_equal(all_repos.status_code, 200)
        self.check_in(repo_expected, all_repos.json())



from lib import BaseTest


class SearchSnapshot1Test(BaseTest):
    """
    search snapshot: regular search
    """
    fixtureDB = True
    fixtureCmds = ["aptly snapshot create wheezy-main from mirror wheezy-main"]
    runCmd = "aptly snapshot search wheezy-main '$$Architecture (i386), Name (% *-dev)'"


class SearchSnapshot2Test(BaseTest):
    """
    search snapshot: missing snapshot
    """
    runCmd = "aptly snapshot search snapshot-xx 'Name'"
    expectedCode = 1


class SearchSnapshot3Test(BaseTest):
    """
    search snapshot: wrong expression
    """
    fixtureDB = True
    fixtureCmds = ["aptly snapshot create wheezy-main from mirror wheezy-main"]
    expectedCode = 1
    runCmd = "aptly snapshot search wheezy-main '$$Architecture (i386'"


class SearchSnapshot4Test(BaseTest):
    """
    search snapshot: with-deps search
    """
    fixtureDB = True
    fixtureCmds = ["aptly snapshot create wheezy-main from mirror wheezy-main"]
    runCmd = "aptly snapshot search -with-deps wheezy-main 'Name (nginx)'"


class SearchSnapshot5Test(BaseTest):
    """
    search snapshot: no results
    """
    fixtureDB = True
    fixtureCmds = ["aptly snapshot create wheezy-main from mirror wheezy-main"]
    runCmd = "aptly snapshot search -with-deps wheezy-main 'Name (no-such-package)'"
    expectedCode = 1


class SearchSnapshot6Test(BaseTest):
    """
    search snapshot: with format
    """
    fixtureDB = True
    fixtureCmds = ["aptly snapshot create wheezy-main from mirror wheezy-main"]
    runCmd = "aptly snapshot search -format='{{.Package}}#{{.Version}}' wheezy-main '$$Architecture (i386), Name (% *-dev)'"


class SearchSnapshot7Test(BaseTest):
    """
    search snapshot: without query
    """
    fixtureDB = True
    fixtureCmds = ["aptly snapshot create wheezy-main from mirror wheezy-main"]
    runCmd = "aptly snapshot search -format='{{.Package}}#{{.Version}}' wheezy-main"



from s3_lib import S3Test


def strip_processor(output):
    return "\n".join([l for l in output.split("\n") if not l.startswith(' ') and not l.startswith('Date:')])


class S3Publish1Test(S3Test):
    """
    publish to S3: from repo
    """
    fixtureCmds = [
        "aptly repo create -distribution=maverick local-repo",
        "aptly repo add local-repo ${files}",
        "aptly repo remove local-repo libboost-program-options-dev_1.62.0.1_i386",
    ]
    runCmd = "aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec local-repo s3:test1:"

    def check(self):
        super(S3Publish1Test, self).check()

        self.check_exists('public/dists/maverick/InRelease')
        self.check_exists('public/dists/maverick/Release')
        self.check_exists('public/dists/maverick/Release.gpg')

        self.check_exists('public/dists/maverick/main/binary-i386/Packages')
        self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
        self.check_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_exists('public/dists/maverick/main/source/Sources')
        self.check_exists('public/dists/maverick/main/source/Sources.gz')
        self.check_exists('public/dists/maverick/main/source/Sources.bz2')

        self.check_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.dsc')
        self.check_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.diff.gz')
        self.check_exists('public/pool/main/p/pyspi/pyspi_0.6.1.orig.tar.gz')
        self.check_exists('public/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc')
        self.check_exists('public/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb')

        # # verify contents except of sums
        self.check_file_contents('public/dists/maverick/Release', 'release', match_prepare=strip_processor)
        self.check_file_contents('public/dists/maverick/main/source/Sources', 'sources', match_prepare=lambda s: "\n".join(sorted(s.split("\n"))))
        self.check_file_contents('public/dists/maverick/main/binary-i386/Packages', 'binary', match_prepare=lambda s: "\n".join(sorted(s.split("\n"))))


class S3Publish2Test(S3Test):
    """
    publish to S3: publish update removed some packages
    """
    fixtureCmds = [
        "aptly repo create -distribution=maverick local-repo",
        "aptly repo add local-repo ${files}/",
        "aptly repo remove local-repo libboost-program-options-dev_1.62.0.1_i386",
        "aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec local-repo s3:test1:",
        "aptly repo remove local-repo pyspi"
    ]
    runCmd = "aptly publish update -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec maverick s3:test1:"

    def check(self):
        super(S3Publish2Test, self).check()

        self.check_exists('public/dists/maverick/InRelease')
        self.check_exists('public/dists/maverick/Release')
        self.check_exists('public/dists/maverick/Release.gpg')

        self.check_exists('public/dists/maverick/main/binary-i386/Packages')
        self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
        self.check_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_exists('public/dists/maverick/main/source/Sources')
        self.check_exists('public/dists/maverick/main/source/Sources.gz')
        self.check_exists('public/dists/maverick/main/source/Sources.bz2')

        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.dsc')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.diff.gz')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1.orig.tar.gz')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc')
        self.check_exists('public/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb')

        # verify contents except of sums
        self.check_file_contents('public/dists/maverick/Release', 'release', match_prepare=strip_processor)
        self.check_file_contents('public/dists/maverick/main/source/Sources', 'sources', match_prepare=lambda s: "\n".join(sorted(s.split("\n"))))
        self.check_file_contents('public/dists/maverick/main/binary-i386/Packages', 'binary', match_prepare=lambda s: "\n".join(sorted(s.split("\n"))))


class S3Publish3Test(S3Test):
    """
    publish to S3: publish switch - removed some packages
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly snapshot create snap2 empty",
        "aptly snapshot pull -no-deps -architectures=i386,amd64 snap2 snap1 snap3 gnuplot-x11",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick snap1 s3:test1:",
    ]
    runCmd = "aptly publish switch -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec maverick s3:test1: snap3"

    def check(self):
        super(S3Publish3Test, self).check()

        self.check_exists('public/dists/maverick/InRelease')
        self.check_exists('public/dists/maverick/Release')
        self.check_exists('public/dists/maverick/Release.gpg')

        self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
        self.check_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_exists('public/dists/maverick/main/binary-amd64/Packages')
        self.check_exists('public/dists/maverick/main/binary-amd64/Packages.gz')
        self.check_exists('public/dists/maverick/main/binary-amd64/Packages.bz2')

        self.check_exists('public/pool/main/g/gnuplot/gnuplot-x11_4.6.1-1~maverick2_i386.deb')
        self.check_exists('public/pool/main/g/gnuplot/gnuplot-x11_4.6.1-1~maverick2_amd64.deb')
        self.check_not_exists('public/pool/main/g/gnuplot/gnuplot-nox_4.6.1-1~maverick2_i386.deb')
        self.check_not_exists('public/pool/main/g/gnuplot/gnuplot-nox_4.6.1-1~maverick2_amd64.deb')

        # verify contents except of sums
        self.check_file_contents('public/dists/maverick/Release', 'release', match_prepare=strip_processor)
        self.check_file_contents('public/dists/maverick/main/binary-i386/Packages', 'binary', match_prepare=lambda s: "\n".join(sorted(s.split("\n"))))


class S3Publish4Test(S3Test):
    """
    publish to S3: multiple repos, list
    """
    fixtureCmds = [
        "aptly repo create -distribution=maverick local-repo",
        "aptly repo add local-repo ${udebs}",
        "aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec local-repo s3:test1:",
        "aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=xyz local-repo s3:test1:",
        "aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec local-repo s3:test1:prefix",
    ]
    runCmd = "aptly publish list"


class S3Publish5Test(S3Test):
    """
    publish to S3: publish drop - component cleanup
    """
    fixtureCmds = [
        "aptly repo create local1",
        "aptly repo create local2",
        "aptly repo add local1 ${files}/libboost-program-options-dev_1.49.0.1_i386.deb",
        "aptly repo add local2 ${files}",
        "aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=sq1 local1 s3:test1:",
        "aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=sq2 local2 s3:test1:",
    ]
    runCmd = "aptly publish drop sq2 s3:test1:"

    def check(self):
        super(S3Publish5Test, self).check()

        self.check_exists('public/dists/sq1')
        self.check_not_exists('public/dists/sq2')
        self.check_exists('public/pool/main/')

        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.dsc')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.diff.gz')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1.orig.tar.gz')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc')
        self.check_exists('public/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb')


class S3Publish6Test(S3Test):
    """
    publish to S3: publish update removed some packages with SSE AES256
    """
    s3Overrides = {'encryptionMethod': 'AES256'}
    fixtureCmds = [
        "aptly repo create -distribution=maverick local-repo",
        "aptly repo add local-repo ${files}/",
        "aptly repo remove local-repo libboost-program-options-dev_1.62.0.1_i386",
        "aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec local-repo s3:test1:",
        "aptly repo remove local-repo pyspi"
    ]
    runCmd = "aptly publish update -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec maverick s3:test1:"

    def check(self):
        super(S3Publish6Test, self).check()

        self.check_exists('public/dists/maverick/InRelease')
        self.check_exists('public/dists/maverick/Release')
        self.check_exists('public/dists/maverick/Release.gpg')

        self.check_exists('public/dists/maverick/main/binary-i386/Packages')
        self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
        self.check_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
        self.check_exists('public/dists/maverick/main/source/Sources')
        self.check_exists('public/dists/maverick/main/source/Sources.gz')
        self.check_exists('public/dists/maverick/main/source/Sources.bz2')

        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.dsc')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.diff.gz')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi_0.6.1.orig.tar.gz')
        self.check_not_exists('public/pool/main/p/pyspi/pyspi-0.6.1-1.3.stripped.dsc')
        self.check_exists('public/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb')

        # verify contents except of sums
        self.check_file_contents('public/dists/maverick/Release', 'release', match_prepare=strip_processor)
        self.check_file_contents('public/dists/maverick/main/source/Sources', 'sources', match_prepare=lambda s: "\n".join(sorted(s.split("\n"))))
        self.check_file_contents('public/dists/maverick/main/binary-i386/Packages', 'binary', match_prepare=lambda s: "\n".join(sorted(s.split("\n"))))



from api_lib import APITest


class MirrorsAPITestCreateShow(APITest):
    """
    POST /api/mirrors, GET /api/mirrors/:name/packages
    """

    def check(self):
        mirror_name = self.random_name()
        mirror_desc = {'Name': mirror_name,
                       'ArchiveURL': 'http://repo.aptly.info/system-tests/security.debian.org/debian-security/',
                       'Architectures': ['amd64'],
                       'Components': ['main'],
                       'Distribution': 'buster/updates'}

        resp = self.post("/api/mirrors", json=mirror_desc)
        self.check_equal(resp.status_code, 400)
        self.check_equal({
            'error': 'unable to fetch mirror: verification of detached signature failed: exit status 2',
        }, resp.json())

        mirror_desc['IgnoreSignatures'] = True
        resp = self.post("/api/mirrors", json=mirror_desc)
        self.check_equal(resp.status_code, 201)

        resp = self.get("/api/mirrors/" + mirror_name)
        self.check_equal(resp.status_code, 200)
        self.check_subset({'Name': mirror_name,
                           'ArchiveRoot': 'http://repo.aptly.info/system-tests/security.debian.org/debian-security/',
                           'Architectures': ['amd64'],
                           'Components': ['main'],
                           'Distribution': 'buster/updates'}, resp.json())

        resp = self.get("/api/mirrors/" + mirror_desc["Name"] + "/packages")
        self.check_equal(resp.status_code, 404)


class MirrorsAPITestCreateUpdate(APITest):
    """
    POST /api/mirrors, PUT /api/mirrors/:name, GET /api/mirrors/:name/packages
    """
    def check(self):
        mirror_name = self.random_name()
        mirror_desc = {'Name': mirror_name,
                       'ArchiveURL': 'http://repo.aptly.info/system-tests/packagecloud.io/varnishcache/varnish30/debian/',
                       'Distribution': 'wheezy',
                       'Keyrings': ["aptlytest.gpg"],
                       'Architectures': ["amd64"],
                       'Components': ['main']}

        mirror_desc['IgnoreSignatures'] = True
        resp = self.post("/api/mirrors", json=mirror_desc)
        self.check_equal(resp.status_code, 201)

        resp = self.get("/api/mirrors/" + mirror_name + "/packages")
        self.check_equal(resp.status_code, 404)

        mirror_desc["Name"] = self.random_name()
        resp = self.put_task("/api/mirrors/" + mirror_name, json=mirror_desc)
        self.check_task(resp)
        _id = resp.json()['ID']

        resp = self.get("/api/tasks/" + str(_id) + "/detail")
        self.check_equal(resp.status_code, 200)
        self.check_equal(resp.json()['RemainingDownloadSize'], 0)
        self.check_equal(resp.json()['RemainingNumberOfPackages'], 0)

        resp = self.get("/api/mirrors/" + mirror_desc["Name"])
        self.check_equal(resp.status_code, 200)
        self.check_subset({'Name': mirror_desc["Name"],
                           'ArchiveRoot': 'http://repo.aptly.info/system-tests/packagecloud.io/varnishcache/varnish30/debian/',
                           'Distribution': 'wheezy'}, resp.json())

        resp = self.get("/api/mirrors/" + mirror_desc["Name"] + "/packages")
        self.check_equal(resp.status_code, 200)


class MirrorsAPITestCreateDelete(APITest):
    """
    POST /api/mirrors, DELETE /api/mirrors/:name
    """
    def check(self):
        mirror_name = self.random_name()
        mirror_desc = {'Name': mirror_name,
                       'ArchiveURL': 'http://repo.aptly.info/system-tests/packagecloud.io/varnishcache/varnish30/debian/',
                       'IgnoreSignatures': True,
                       'Distribution': 'wheezy',
                       'Components': ['main']}

        resp = self.post("/api/mirrors", json=mirror_desc)
        self.check_equal(resp.status_code, 201)

        resp = self.delete_task("/api/mirrors/" + mirror_name)
        self.check_task(resp)


class MirrorsAPITestCreateList(APITest):
    """
    GET /api/mirrors, POST /api/mirrors, GET /api/mirrors
    """
    def check(self):
        resp = self.get("/api/mirrors")
        self.check_equal(resp.status_code, 200)
        count = len(resp.json())

        mirror_name = self.random_name()
        mirror_desc = {'Name': mirror_name,
                       'ArchiveURL': 'http://repo.aptly.info/system-tests/packagecloud.io/varnishcache/varnish30/debian/',
                       'IgnoreSignatures': True,
                       'Distribution': 'wheezy',
                       'Components': ['main']}

        resp = self.post("/api/mirrors", json=mirror_desc)
        self.check_equal(resp.status_code, 201)

        resp = self.get("/api/mirrors")
        self.check_equal(resp.status_code, 200)
        self.check_equal(len(resp.json()), count + 1)


class MirrorsAPITestSkipArchitectureCheck(APITest):
    """
    GET /api/mirrors, POST /api/mirrors, GET /api/mirrors

    This tests SkipArchitectureCheck and IgnoreSignatures via API.
    The repo to be mirrored requires the SkipArchitectureCheck and SkipComponentCheck in order to be mirrored.
    """
    def check(self):
        resp = self.get("/api/mirrors")
        self.check_equal(resp.status_code, 200)
        count = len(resp.json())

        mirror_name = self.random_name()
        mirror_desc = {'Name': mirror_name,
                       'ArchiveURL': 'http://repo.aptly.info/system-tests/pkg.duosecurity.com/Debian',
                       'Architectures': ['amd64', 'i386'],
                       'SkipArchitectureCheck': True,
                       'SkipComponentCheck': True,
                       'IgnoreSignatures': True,
                       'Distribution': 'bookworm',
                       'Components': ['main']}

        resp = self.post("/api/mirrors", json=mirror_desc)
        self.check_equal(resp.status_code, 201)

        resp = self.get("/api/mirrors")
        self.check_equal(resp.status_code, 200)
        self.check_equal(len(resp.json()), count + 1)

        mirror_desc = {'Name': mirror_name,
                       'IgnoreSignatures': True}
        resp = self.put_task("/api/mirrors/" + mirror_name, json=mirror_desc)
        self.check_task(resp)



from lib import BaseTest


class RenameSnapshot1Test(BaseTest):
    """
    rename snapshot: regular operations
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snapshot1 from mirror wheezy-contrib",
    ]
    runCmd = "aptly snapshot rename snapshot1 snapshot2"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly snapshot list", "snapshot_list")


class RenameSnapshot2Test(BaseTest):
    """
    rename snapshot: missing snapshot
    """
    runCmd = "aptly snapshot rename snapshot2 snapshot3"
    expectedCode = 1


class RenameSnapshot3Test(BaseTest):
    """
    rename snapshot: already exists
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snapshot3 from mirror wheezy-contrib",
        "aptly snapshot create snapshot4 from mirror wheezy-contrib",
    ]
    runCmd = "aptly snapshot rename snapshot3 snapshot4"
    expectedCode = 1



from lib import BaseTest


class DropSnapshot1Test(BaseTest):
    """
    drop snapshot: just drop
    """
    fixtureDB = True
    fixtureCmds = ["aptly snapshot create snap1 from mirror wheezy-non-free"]
    runCmd = "aptly snapshot drop snap1"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly snapshot show snap1", "snapshot_show", expected_code=1)


class DropSnapshot2Test(BaseTest):
    """
    drop snapshot: used as source
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-non-free",
        "aptly snapshot merge snap2 snap1",
    ]
    runCmd = "aptly snapshot drop snap1"
    expectedCode = 1


class DropSnapshot3Test(BaseTest):
    """
    drop snapshot: -force
    """
    fixtureDB = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror wheezy-non-free",
        "aptly snapshot merge snap2 snap1",
    ]
    runCmd = "aptly snapshot drop -force snap1"

    def check(self):
        self.check_output()
        self.check_cmd_output("aptly snapshot show snap1", "snapshot_show", expected_code=1)


class DropSnapshot4Test(BaseTest):
    """
    drop snapshot: already published
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1"
    ]
    runCmd = "aptly snapshot drop snap1"
    expectedCode = 1


class DropSnapshot5Test(BaseTest):
    """
    drop snapshot: already published with -force
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1"
    ]
    runCmd = "aptly snapshot drop --force snap1"
    expectedCode = 1


class DropSnapshot6Test(BaseTest):
    """
    drop snapshot: no such snapshot
    """
    fixtureDB = True
    runCmd = "aptly snapshot drop no-such-snapshot"
    expectedCode = 1


class DropSnapshot7Test(BaseTest):
    """
    drop snapshot: publish, drop publish, drop snapshot
    """
    fixtureDB = True
    fixturePool = True
    fixtureCmds = [
        "aptly snapshot create snap1 from mirror gnuplot-maverick",
        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1",
        "aptly publish drop maverick",
    ]
    runCmd = "aptly snapshot drop snap1"



from lib import BaseTest


class RunTask1Test(BaseTest):
    """
    task run: simple commands, 1-word command
    """
    gold_processor = BaseTest.expand_environ

    runCmd = "aptly task run repo list, repo create local, repo drop local, version"


class RunTask2Test(BaseTest):
    """
    task run: commands with args
    """
    runCmd = "aptly task run -- repo list -raw, repo create local, repo list"


class RunTask3Test(BaseTest):
    """
    task run: failure
    """
    expectedCode = 1
    runCmd = "aptly task run -- repo show a, repo create local, repo list"


class RunTask4Test(BaseTest):
    """
    task run: from file
    """
    runCmd = "aptly task run -filename=${testfiles}/task"


class RunTask5Test(BaseTest):
    """
    task run: from file not found
    """
    expectedCode = 1
    runCmd = "aptly task run -filename=not_found"



from lib import BaseTest
import uuid
import os

try:
    from azure.storage.blob import BlobServiceClient

    azure_storage_account = os.environ.get('AZURE_STORAGE_ACCOUNT')
    azure_storage_access_key = os.environ.get('AZURE_STORAGE_ACCESS_KEY')
    azure_storage_endpoint = os.environ.get(
        'AZURE_STORAGE_ENDPOINT',
        f'https://{azure_storage_account}.blob.core.windows.net',
    )
    if azure_storage_account is not None and azure_storage_access_key is not None:
        blob_client = BlobServiceClient(
            account_url=azure_storage_endpoint,
            credential=azure_storage_access_key,
        )
    else:
        print('Azure tests disabled: Azure creds not found in the environment')
        blob_client = None
except ImportError as e:
    print("Azure tests disabled: can't import azure.storage.blob", e)
    blob_client = None


class AzureTest(BaseTest):
    """
    BaseTest + support for Azure Blob Storage
    """

    use_azure_pool = False

    def __init__(self) -> None:
        super(AzureTest, self).__init__()
        self.container_name = None
        self.container = None
        self.container_contents = None

    def fixture_available(self):
        return super(AzureTest, self).fixture_available() and blob_client is not None

    def prepare(self):
        self.container_name = 'aptly-sys-test-' + str(uuid.uuid1())
        self.container = blob_client.create_container(
            self.container_name, public_access='blob'
        )

        self.azure_endpoint = {
            'accountName': azure_storage_account,
            'accountKey': azure_storage_access_key,
            'container': self.container_name,
            'endpoint': azure_storage_endpoint,
        }

        self.configOverride = {
            'AzurePublishEndpoints': {
                'test1': self.azure_endpoint,
            },
        }
        if self.use_azure_pool:
            self.configOverride['packagePoolStorage'] = {
                'type': 'azure',
                **self.azure_endpoint,
            }

        super(AzureTest, self).prepare()

    def shutdown(self):
        if self.container_name is not None:
            blob_client.delete_container(self.container_name)

        super(AzureTest, self).shutdown()

    def check_path(self, path):
        if self.container_contents is None:
            self.container_contents = [
                p.name for p in self.container.list_blobs() if p.name is not None
            ]

        if path.startswith('public/'):
            path = path.removeprefix('public/')

        if path in self.container_contents:
            return True

        if not path.endswith('/'):
            path = path + '/'

        for item in self.container_contents:
            if item.startswith(path):
                return True

        return False

    def check_exists(self, path):
        if not self.check_path(path):
            raise Exception("path %s doesn't exist" % (path,))

    def check_exists_azure_only(self, path):
        self.check_exists(path)
        BaseTest.check_not_exists(self, path)

    def check_not_exists(self, path):
        if self.check_path(path):
            raise Exception('path %s exists' % (path,))

    def read_file(self, path, mode=''):
        assert not mode

        if path.startswith('public/'):
            path = path.removeprefix('public/')

        blob = self.container.download_blob(path)
        return blob.readall().decode('utf-8')



"""
Test help screens
"""

import re
from lib import BaseTest


class MainTest(BaseTest):
    """
    main
    """
    expectedCode = 2
    runCmd = "aptly"

    def outputMatchPrepare(_, s):
        return re.sub(r'  -(cpuprofile|memprofile|memstats|meminterval)=.*\n', '', s, flags=re.MULTILINE)


class MirrorTest(BaseTest):
    """
    main
    """
    expectedCode = 2
    runCmd = "aptly mirror"


class MirrorCreateTest(BaseTest):
    """
    main
    """
    expectedCode = 2
    runCmd = "aptly mirror create"


class MainHelpTest(BaseTest):
    """
    main
    """
    runCmd = "aptly help"

    def outputMatchPrepare(_, s):
        return re.sub(r'  -(cpuprofile|memprofile|memstats|meminterval)=.*\n', '', s, flags=re.MULTILINE)


class MirrorHelpTest(BaseTest):
    """
    main
    """
    runCmd = "aptly help mirror"


class MirrorCreateHelpTest(BaseTest):
    """
    main
    """
    runCmd = "aptly help mirror create"


class WrongFlagTest(BaseTest):
    """
    main
    """
    expectedCode = 2
    runCmd = "aptly mirror create -fxz=sss"
